using Microsoft.VisualBasic;
using System;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;

using System.Collections.Generic;
using System.Threading;
using System.IO;
using System.Media;
namespace Theremino_Dolfrang
{

	public partial class Form_Main
	{
		// ------------------------------------------------------------------------- PROCESSES
		private Thread AcquisitionThread;
		private Stopwatch AcquisitionTimer = new Stopwatch();
		// ------------------------------------------------------------------------- ACQUISITION
		private bool WriteDataFile;
		// ------------------------------------------------------------------------- 
		private float SamplingFreq_hz;
		private Int32 SamplingTime_ms;
		private Int32 NumChannels;
		private static Int32 NumChannels_Max = 25;
		private Int32 ZeroingTime_ms;
		private Int32 AcquisitionTime_ms;
		// ------------------------------------------------------------------------- 
		private float VisualizationFreq_hz;
		private Int32 VisualizationTime_ms = 2;
		private Int32 ZoomX;
		private Int32 ZoomY;
		// ------------------------------------------------------------------------- 
		private float[] MidValue = new float[NumChannels_Max];
		private float[] LastValue = new float[NumChannels_Max];
		private float[] OldValue = new float[NumChannels_Max];
		// ------------------------------------------------------------------------- 
		private string AcquisitionSystem;
		private string TransducerType;
		private string SurveyName;
		private string Department;
		private string Locality;
		private string Notes;
		private string StationCode;
		private Int32 VoltageGain;
		private string OutDataFormat;
		private string IdChannel1;
		private string IdChannel2;
		private string IdChannel3;
		private string Rotation;
		// ------------------------------------------------------------------------- DRAWING
		private Int32 Operation;
		private Int32 Percentual;
		private bool UpdatingGraph = false;
		private Graphics GFX;
		private Pen[] PenColors = new Pen[25];
		private Int32 x;
		private int ReadSamplesTotal;

		private bool StopAcquisitionFlag;
		// ===============================================================
		//   MAIN FORM
		// ===============================================================   
        public Form_Main()
        {
            LocationChanged += Form_LocationChanged;
            Resize += Form1_Resize;
            FormClosing += Form1_FormClosing;
            Load += Form1_Load;
            InitializeComponent();
        }

        static bool InitStaticVariableHelper(Microsoft.VisualBasic.CompilerServices.StaticLocalInitFlag flag)
        {
            if (flag.State == 0)
            {
                flag.State = 2;
                return true;
            }
            else if (flag.State == 2)
            {
                throw new Microsoft.VisualBasic.CompilerServices.IncompleteInitialization();
            }
            else
            {
                return false;
            }
        }

		private void Form1_Load(object sender, System.EventArgs e)
		{
			//----------------------------------------------------------------------
            Text = Module_SaveLoad.AppTitleAndVersion("Theremino Dolfrang - CSharp");
			//---------------------------------------------------------------------- LOAD INITIAL PARAMS
            Module_SaveLoad.Load_INI();
            Utils_LocaleNames.SetLocales();
			//ToolTips_Init()
			ToolStrip1.Renderer = new ToolStripButtonRenderer();
            ShowKvalues();
			//---------------------------------------------------------------------- START HAL 
			StartThereminoHAL();
			// --------------------------------------------------------------------- COLORS
			PenColors[0] = new Pen(Brushes.Red, 1);
			PenColors[1] = new Pen(Brushes.Red, 1);
			PenColors[2] = new Pen(Brushes.Green);
			PenColors[3] = new Pen(Brushes.Blue);
			PenColors[4] = new Pen(Brushes.Red);
			PenColors[5] = new Pen(Brushes.Green);
			PenColors[6] = new Pen(Brushes.Blue);
			PenColors[7] = new Pen(Brushes.Red);
			PenColors[8] = new Pen(Brushes.Green);
			PenColors[9] = new Pen(Brushes.Blue);
			PenColors[10] = new Pen(Brushes.Red);
			PenColors[11] = new Pen(Brushes.Green);
			PenColors[12] = new Pen(Brushes.Blue);
			PenColors[13] = new Pen(Brushes.Red);
			PenColors[14] = new Pen(Brushes.Green);
			PenColors[15] = new Pen(Brushes.Blue);
			PenColors[16] = new Pen(Brushes.Red);
			PenColors[17] = new Pen(Brushes.Green);
			PenColors[18] = new Pen(Brushes.Blue);
			PenColors[19] = new Pen(Brushes.Red);
			PenColors[20] = new Pen(Brushes.Green);
			PenColors[21] = new Pen(Brushes.Blue);
			PenColors[22] = new Pen(Brushes.Red);
			PenColors[23] = new Pen(Brushes.Green);
			PenColors[24] = new Pen(Brushes.Blue);
			// -------------------------------------------------------------------------------
			// REAL TIME REQUESTING
			// -------------------------------------------------------------------------------
			System.Threading.Thread.CurrentThread.Priority = System.Threading.ThreadPriority.Normal;
			System.Diagnostics.Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.RealTime;
			// -------------------------------------------------------------------------------
			UserInterfaceTimer.Start();
            Module_SaveLoad.EventsAreEnabled = true;
            // ------------------------------------------- To be removed in the next version
            TabPage2.Dispose();
			// ------------------------------------------- FadeIn
			Refresh();
            Module_SaveLoad.Forms_FadeTo(1, 400);
		}
		private void Form1_FormClosing(object sender, System.Windows.Forms.FormClosingEventArgs e)
		{
            Module_SaveLoad.Forms_FadeTo(0, 500);
			this.Refresh();
            Module_SaveLoad.Save_INI();
			StopThereminoHAL();
		}
		private void Form1_Resize(object sender, System.EventArgs e)
		{
			InitPictureboxImage();
		}
		private void Form_LocationChanged(object sender, System.EventArgs e)
		{
			if (!Module_SaveLoad.EventsAreEnabled)
				return;
            Module_SaveLoad.LimitFormPosition(this);
		}

		// ===================================================================================
		//  MenuStrip and ToolStrip accepting the first click
		//  If the form receives a WM_PARENTNOTIFY (528) message and is not focused 
		//  then the form is activated before to execute the message
		// ===================================================================================
		protected override void WndProc(ref Message m)
		{
			if (!this.Focused && m.Msg == 528) 
            {
				this.Activate();
			}
			base.WndProc(ref m);
		}

		// ===================================================================================
		//   MenuStrip and ToolStrip Gradients
		// ===================================================================================
		private void MenuStrip1_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			Rectangle bounds = new Rectangle(0, 0, MenuStrip1.Width, MenuStrip1.Height);
			System.Drawing.Drawing2D.LinearGradientBrush brush = new System.Drawing.Drawing2D.LinearGradientBrush(bounds, Color.FromArgb(240, 240, 240), Color.FromArgb(200, 200, 200), System.Drawing.Drawing2D.LinearGradientMode.Horizontal);
			e.Graphics.FillRectangle(brush, bounds);
		}
		private void ToolStrip1_Paint(System.Object sender, System.Windows.Forms.PaintEventArgs e)
		{
			Rectangle bounds = new Rectangle(0, 0, ToolStrip1.Width, ToolStrip1.Height);
			System.Drawing.Drawing2D.LinearGradientBrush brush = new System.Drawing.Drawing2D.LinearGradientBrush(bounds, Color.White, Color.FromArgb(200, 200, 200), System.Drawing.Drawing2D.LinearGradientMode.Vertical);
			e.Graphics.FillRectangle(brush, bounds);
		}


		// ===================================================================================
		//   ToolStrip PressedButton color
		// ===================================================================================
		public class ToolStripButtonRenderer : System.Windows.Forms.ToolStripProfessionalRenderer
		{
			protected override void OnRenderButtonBackground(ToolStripItemRenderEventArgs e)
			{
				ToolStripButton btn = (ToolStripButton)e.Item;
				if (btn != null && btn.CheckOnClick && btn.Checked) 
                {
					Rectangle bounds = new Rectangle(0, 0, e.Item.Width - 1, e.Item.Height - 1);
					System.Drawing.Drawing2D.LinearGradientBrush brush = new System.Drawing.Drawing2D.LinearGradientBrush(bounds, Color.Gold, Color.FromArgb(255, 255, 255), System.Drawing.Drawing2D.LinearGradientMode.Vertical);
					e.Graphics.FillRectangle(brush, bounds);
					e.Graphics.DrawRectangle(Pens.Orange, bounds);
				} else 
                {
					base.OnRenderButtonBackground(e);
				}
			}
		}


		// =======================================================================================
		//   MENU
		// =======================================================================================
		private void Menu_File_Open_Click(System.Object sender, System.EventArgs e)
		{
			Process.Start(Application.StartupPath);
		}
		private void Menu_File_Exit_Click(System.Object sender, System.EventArgs e)
		{
			this.Close();
		}
		private void Menu_Info_Click(System.Object sender, System.EventArgs e)
		{
			My.MyProject.Forms.Form_About.ShowDialog(this);
		}

		// =======================================================================================
		//   MENU LANGUAGE
		// =======================================================================================
		private void Menu_Language_DropDownOpening(object sender, System.EventArgs e)
		{
			foreach (ToolStripMenuItem item in Menu_Language.DropDownItems) 
            {
                if (item.Name.EndsWith(Utils_LocaleNames.Language, StringComparison.InvariantCultureIgnoreCase))
                {
					item.Select();
				}
			}
		}
		private void Menu_Language_Deutsch_Click(object sender, System.EventArgs e)
		{
            Utils_LocaleNames.Language = "Deutsch";
            Utils_LocaleNames.SetLocales();
			Module_SaveLoad.Save_INI();
			//ToolTips_Init()
		}
		private void Menu_Language_English_Click(object sender, System.EventArgs e)
		{
            Utils_LocaleNames.Language = "English";
            Utils_LocaleNames.SetLocales();
            Module_SaveLoad.Save_INI();
			//ToolTips_Init()
		}
		private void Menu_Language_Espanol_Click(object sender, System.EventArgs e)
		{
            Utils_LocaleNames.Language = "Espanol";
            Utils_LocaleNames.SetLocales();
            Module_SaveLoad.Save_INI();
			//ToolTips_Init()
		}
		private void Menu_Language_Francais_Click(object sender, System.EventArgs e)
		{
            Utils_LocaleNames.Language = "Francais";
            Utils_LocaleNames.SetLocales();
            Module_SaveLoad.Save_INI();
			//ToolTips_Init()
		}
		private void Menu_Language_Italian_Click(object sender, System.EventArgs e)
		{
            Utils_LocaleNames.Language = "Italian";
            Utils_LocaleNames.SetLocales();
            Module_SaveLoad.Save_INI();
			//ToolTips_Init()
		}
		private void Menu_Language_Japanese_Click(System.Object sender, System.EventArgs e)
		{
            Utils_LocaleNames.Language = "Japanese";
            Utils_LocaleNames.SetLocales();
            Module_SaveLoad.Save_INI();
			//ToolTips_Init()
		}


		// =======================================================================================
		//   TOOLBAR
		// =======================================================================================
		private void ToolStripButton_OpenProgramFolder_Click(System.Object sender, System.EventArgs e)
		{
			Process.Start(Application.StartupPath);
			ToolStripButton_OpenFolder.Checked = false;
		}
		private void ToolStripButton_StartAcquisition_Click(System.Object sender, System.EventArgs e)
		{
            TabControl1.SelectedIndex = 1;  // 2 if trimming page is visible
			Refresh();
			StartAcquisition();
			ToolStripButton_Start.Checked = false;
		}
		private void ToolStripButton_StopAcquisition_Click(System.Object sender, System.EventArgs e)
		{
			StopAcquisitionFlag = true;
			ToolStripButton_Stop.Checked = false;
		}


		// =========================================================================
		//   THEREMINO HAL - START STOP -
		// =========================================================================
		private Process HalProcess = null;
		private void StartThereminoHAL()
		{
			string path = Application.StartupPath + "\\Theremino_HAL.exe";
			if (!Theremino_Dolfrang.My.MyProject.Computer.FileSystem.FileExists(path)) 
            {
				path = Application.StartupPath + "\\Theremino_HAL\\Theremino_HAL.exe";
			}
			if (!Theremino_Dolfrang.My.MyProject.Computer.FileSystem.FileExists(path)) return;
			ProcessStartInfo psi = new ProcessStartInfo();
			psi.WorkingDirectory = System.IO.Path.GetDirectoryName(path);
			psi.FileName = path;
			HalProcess = Process.Start(psi);
		}
		private void StopThereminoHAL()
		{
			if (HalProcess != null && !HalProcess.HasExited) 
            {
				try 
                {
					HalProcess.CloseMainWindow();
				} 
                finally 
                {
				}
			}
		}


		// ===============================================================
		//    MAIN BITMAP
		// =============================================================== 
		private void InitPictureboxImage()
		{
			var _with1 = PictureBox1;
			if (_with1.ClientSize.Height < 1) return;
			if (_with1.Image == null || _with1.Image.Width != _with1.ClientSize.Width || _with1.Image.Height != _with1.ClientSize.Height) 
            {
				_with1.Image = new Bitmap(_with1.ClientRectangle.Width, _with1.ClientRectangle.Height, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
			}
			GFX = Graphics.FromImage(PictureBox1.Image);
			GFX.Clear(PictureBox1.BackColor);
			_with1.Refresh();
		}


		// ===============================================================
		//    WINDOW TIMERS PRECISION
		// =============================================================== 
        [System.Runtime.InteropServices.DllImport("winmm.dll")]
        private static extern Int32 timeBeginPeriod(Int32 uPeriod);
        [System.Runtime.InteropServices.DllImport("winmm.dll")]
        private static extern Int32 timeEndPeriod(Int32 uPeriod);
		private void MillisecondPrecision()
		{
			timeBeginPeriod(1);
		}
		private void NormalPrecision()
		{
			timeEndPeriod(1);
		}
		readonly Microsoft.VisualBasic.CompilerServices.StaticLocalInitFlag static_UserInterfaceTimer_Tick_oldoperation_Init = new Microsoft.VisualBasic.CompilerServices.StaticLocalInitFlag();


		// ===============================================================
		//   USER INTERFACE TIMER
		// =============================================================== 
		Int32 static_UserInterfaceTimer_Tick_oldoperation;

		Int32 static_UserInterfaceTimer_Tick_oldpercentual;
		private void UserInterfaceTimer_Tick(System.Object sender, System.EventArgs e)
		{
			lock (static_UserInterfaceTimer_Tick_oldoperation_Init) 
            {
				try 
                {
					if (InitStaticVariableHelper(static_UserInterfaceTimer_Tick_oldoperation_Init)) 
                    {
						static_UserInterfaceTimer_Tick_oldoperation = -1;
					}
				} 
                finally 
                {
					static_UserInterfaceTimer_Tick_oldoperation_Init.State = 1;
				}
			}
			if (Operation != static_UserInterfaceTimer_Tick_oldoperation) 
            {
				static_UserInterfaceTimer_Tick_oldoperation = Operation;
				ToolStripStatusLabel1.Text = OperationName(Operation);
				switch (Operation) 
                {
					case 0:
						ToolStripButton_Start.Enabled = true;
						ToolStripButton_Stop.Enabled = false;
						ToolStripButton_WriteDataFile.Enabled = true;
						GroupBox_SurveyParams.Enabled = true;
						GroupBox_HardwareParams.Enabled = true;
						GroupBox_AcquisitionParams.Enabled = true;
						GroupBox_ViewParams.Enabled = true;
                        TabPage2.Enabled = true;
						break;
					case 1:
                    case 2:
						ToolStripButton_Start.Enabled = false;
						ToolStripButton_Stop.Enabled = true;
						ToolStripButton_WriteDataFile.Enabled = false;
						GroupBox_SurveyParams.Enabled = false;
						GroupBox_HardwareParams.Enabled = false;
						GroupBox_AcquisitionParams.Enabled = false;
						GroupBox_ViewParams.Enabled = true;
                        TabPage2.Enabled = false;
						break;
				}
			}
			if (Percentual != static_UserInterfaceTimer_Tick_oldpercentual) 
            {
				if (Percentual > 100) Percentual = 100;
				static_UserInterfaceTimer_Tick_oldpercentual = Percentual;
				ToolStripProgressBar1.Value = Percentual;
			}

			if (UpdatingGraph) 
            {
				UpdateGraph();
			}
		}

		private string OperationName(Int32 operation)
		{
			switch (operation) 
            {
				case 0:
					return "";
				case 1:
                    return Utils_LocaleNames.Msg_OperationZeroSetting;
				case 2:
                    return Utils_LocaleNames.Msg_OperationAcquisition;
				default:
					return "";
			}
		}

		private void UpdateGraph()
		{
			// --------------------------------------------------------------------- 
			Int32 w = PictureBox1.Image.Width;
			Int32 h = PictureBox1.Image.Height;
			// --------------------------------------------------------------------- END OF PAGE
			if (x * ZoomX > w)
				x = 0;
			// --------------------------------------------------------------------- 
			Int32 x1 = (x - 1) * ZoomX;
			Int32 x2 = x * ZoomX;
			Int32 y = default(Int32);
            // --------------------------------------------------------------------- Yscale variable with gain
            float ky = 10000.0F * ZoomY / VoltageGain;

			float value = 0;
			// --------------------------------------------------------------------- BACKGROUND CLEARING
			GFX.FillRectangle(new SolidBrush(PictureBox1.BackColor), x1 + 1, 0, x2 + 10, h);
			// --------------------------------------------------------------------- 
			for (Int32 i = 1; i <= NumChannels; i++) 
            {
				value = LastValue[i] * ky;
				y = Convert.ToInt32(i * (h / (NumChannels + 1)));
				GFX.DrawLine(PenColors[i], x1, y + OldValue[i], x2, y + value);
				if (LastValue[i] < 0) 
                {
					GFX.DrawLine(PenColors[i], x2, y, x2, y + value);
				}
				OldValue[i] = value;
			}
			x += 1;
			// --------------------------------------------------------------------- PICTUREBOX UPDATE
			Rectangle rec = default(Rectangle);
			rec.X = x1;
			rec.Width = ZoomX;
			rec.Height = h;
			PictureBox1.Invalidate(rec);  // invalidate only the modified rectangle
		}

		// =======================================================================================
		//   VisualizationFreq not faster then SamplingFreq and not more than 100
		// =======================================================================================
		private void cmb_VisualizationFreq_SelectedIndexChanged(System.Object sender, System.EventArgs e)
		{
			CorrectVisualizationFreq();
            VisualizationParams_Update(null, null);
		}
		private void cmb_SamplingFreq_SelectedIndexChanged(System.Object sender, System.EventArgs e)
		{
			CorrectVisualizationFreq();
		}
		private void CorrectVisualizationFreq()
		{
			if (cmb_VisualizationFreq.SelectedIndex < cmb_SamplingFreq.SelectedIndex - 5) 
            {
				cmb_VisualizationFreq.SelectedIndex = cmb_SamplingFreq.SelectedIndex - 5;
			}
            if (cmb_VisualizationFreq.SelectedIndex > 3)
            {
                lbl_VisualizationFreqNote.ForeColor = Color.DimGray;
            }
            else if (cmb_VisualizationFreq.SelectedIndex > 1)
            {
                lbl_VisualizationFreqNote.ForeColor = Color.DarkRed;
            }
            else
            {
                lbl_VisualizationFreqNote.ForeColor = Color.Red;
            }
		}

		// =======================================================================================
		//   Visualization params update while running
		// =======================================================================================
		private void VisualizationParams_Update(System.Object sender, System.EventArgs e)
        {
            if(!Module_SaveLoad.EventsAreEnabled) return;
            SetVisualizationParams();
        }

		// ===============================================================
		//   SET PARAMS 
		// =============================================================== 
		private void SetSurveyParams()
		{
			AcquisitionSystem = "Theremino Dolfrang";
			SurveyName = txt_SurveyName.Text;
			Locality = txt_Locality.Text;
			Department = txt_Department.Text;
			Notes = txt_Notes.Text;
		}
		private void SetHardwareParams()
		{
			Rotation = txt_Rotation.Text;
			StationCode = txt_StationCode.Text;
			VoltageGain = Convert.ToInt32(Conversion.Val(txt_VoltageGain.Text));
			if (VoltageGain < 1){VoltageGain = 1;txt_VoltageGain.Text = "1";}
			OutDataFormat = cmb_OutDataFormat.Text;
			TransducerType = cmb_TransducerType.Text;
			IdChannel1 = cmb_Channel1.Text;
			IdChannel2 = cmb_Channel2.Text;
			IdChannel3 = cmb_Channel3.Text;
		}
		private void SetAcquisitionParams()
		{
			SamplingFreq_hz = Convert.ToInt32(Conversion.Val(cmb_SamplingFreq.Text.Replace(",", ".")));
			if (SamplingFreq_hz < 1)
				SamplingFreq_hz = 1;
			if (SamplingFreq_hz > 500)
				SamplingFreq_hz = 500;
			SamplingTime_ms = Convert.ToInt32(1000 / SamplingFreq_hz);
			if (SamplingTime_ms < 2)
				SamplingTime_ms = 2;
			// --------------------------------------------------------------------- Num channels
			NumChannels = Convert.ToInt32(Conversion.Val(cmb_NumChannels.Text.Replace(",", ".")));
            // --------------------------------------------------------------------- Zeroing time
			ZeroingTime_ms = 1000 * Convert.ToInt32(Conversion.Val(cmb_ZeroingTime.Text));
			// --------------------------------------------------------------------- Acquisition time
			AcquisitionTime_ms = 1000 * Convert.ToInt32(Conversion.Val(cmb_AcquisitionTime.Text));
			if (cmb_AcquisitionTimeUnit.SelectedIndex == 1) 
            {
				AcquisitionTime_ms *= 60;
			}
		}
		private void SetVisualizationParams()
		{
			// --------------------------------------------------------------------- Visualization frequency and time
			VisualizationFreq_hz = Convert.ToInt32(Conversion.Val(cmb_VisualizationFreq.Text.Replace(",", ".")));
			if (VisualizationFreq_hz < 1)
				VisualizationFreq_hz = 1;
			if (VisualizationFreq_hz > 500)
				VisualizationFreq_hz = 500;
			VisualizationTime_ms = Convert.ToInt32(1000 / VisualizationFreq_hz);
			if (VisualizationTime_ms < 2)
				VisualizationTime_ms = 2;
			UserInterfaceTimer.Interval = VisualizationTime_ms;
			// --------------------------------------------------------------------- Visualization Zoom
			ZoomX = Convert.ToInt32(Conversion.Val(cmb_ZoomX.Text));
			ZoomY = Convert.ToInt32(Conversion.Val(cmb_ZoomY.Text));
		}


		// ===============================================================
		//   START ACQUISITION
		// ===============================================================       
		private void StartAcquisition()
		{
			// --------------------------------------------------------------------- gat all params
			WriteDataFile = ToolStripButton_WriteDataFile.Checked;
			SetSurveyParams();
			SetHardwareParams();
			SetAcquisitionParams();
			SetVisualizationParams();
            SetChannelGains();
			// --------------------------------------------------------------------- 
			InitPictureboxImage();
			// --------------------------------------------------------------------- start acquisition thread
			if (AcquisitionThread == null || !AcquisitionThread.IsAlive) 
            {
				AcquisitionThread = new Thread(ZeroingAndAcquisition);
				AcquisitionThread.IsBackground = true;
				AcquisitionThread.Priority = ThreadPriority.Highest;
				AcquisitionThread.Start();
			}
		}

		private void ZeroingAndAcquisition()
		{
			// ---------------------------------------------------------------------
			Int32 TotalSamples = AcquisitionTime_ms / SamplingTime_ms;
			double ThisTime = 0;
			double NextTime = 0;
			Int32 i = default(Int32);
			// --------------------------------------------------------------------- ZEROING
			MillisecondPrecision();
			AcquisitionTimer.Reset();
			AcquisitionTimer.Start();
			Operation = 1;
            StopAcquisitionFlag = false;
			Interaction.Beep();
			ReadSamplesTotal = 0;
			TotalSamples = Convert.ToInt32(ZeroingTime_ms / SamplingTime_ms);
			ThisTime = 0;
			NextTime = 0;
			// --------------------------------------------------------------------- RESET MEDIA
			for (i = 1; i <= NumChannels; i++) 
            {
				MidValue[i] = 0;
			}
            // --------------------------------------------------------------------- VALUES ACCUMULATION
            if (ZeroingTime_ms > 0)
            {
                AcquisitionTimer.Reset();
                AcquisitionTimer.Start();
                do
                {
                    Percentual = Convert.ToInt32(ThisTime * 100 / ZeroingTime_ms);
                    ReadSamplesTotal += 1;
                    //
                    for (i = 1; i <= NumChannels; i++)
                    {
                        MidValue[i] += ThereminoSlots.ReadSlot_NoNan(i);
                    }
                    //
                    NextTime += SamplingTime_ms;
                    do
                    {
                        System.Threading.Thread.Sleep(1);
                        ThisTime = AcquisitionTimer.Elapsed.TotalMilliseconds;
                    } while (!(ThisTime >= NextTime));
                    //
                } while (!(ReadSamplesTotal >= TotalSamples | StopAcquisitionFlag));
                AcquisitionTimer.Stop();
                Percentual = 0;
                // --------------------------------------------------------------------- MID VALUE CALCULATION
                for (i = 1; i <= NumChannels; i++)
                {
                    MidValue[i] = MidValue[i] / ReadSamplesTotal;
                }
            }
            // --------------------------------------------------------------------- ACQUISITION
            Interaction.Beep();
			Operation = 2;
			TotalSamples = Convert.ToInt32(AcquisitionTime_ms / SamplingTime_ms);
			// --------------------------------------------------------------------- OPEN THE OUTPUT FILE
			string Path = Application.StartupPath + "\\DataFiles\\";
			//Dim Name As String = "Test1.saf"
			string Name = txt_SurveyName.Text + "_" + DateTime.Now.Year + "_" + DateTime.Now.Month + "_" + DateTime.Now.Day + "_" + DateTime.Now.Hour + "_" + DateTime.Now.Minute + ".saf";

			Directory.CreateDirectory(Path);
			// --------------------------------------------------------------------- SAF PARAMS
			StreamWriter File = null;
            try
            {
                // START_TIME=2002 4 26 17 03 12.1233324 (5 integers, 1 real)
                if (WriteDataFile)
                {
                    File = System.IO.File.CreateText(Path + Name);
                    File.WriteLine("SESAME ASCII data format (saf) v. 1");
                    File.WriteLine("# ----------------------------------");
                    File.WriteLine("START_TIME  = " + DateTime.Now.Year + " " + DateTime.Now.Month + " " + DateTime.Now.Day + " " + DateTime.Now.Hour + " " + DateTime.Now.Minute + " " + DateTime.Now.Second + "." + DateTime.Now.Millisecond);
                    File.WriteLine("ACQ_SYSTEM  = " + AcquisitionSystem);
                    File.WriteLine("SURVEY_NAME = " + SurveyName);
                    if (!string.IsNullOrEmpty(Department))
                        File.WriteLine("Department  = " + Department);
                    if (!string.IsNullOrEmpty(Locality))
                        File.WriteLine("Locality    = " + Locality);
                    if (!string.IsNullOrEmpty(Notes))
                        File.WriteLine("Notes       = " + Notes);
                    File.WriteLine("STA_CODE    = " + StationCode);
                    File.WriteLine("SAMP_FREQ   = " + SamplingFreq_hz);
                    File.WriteLine("NDAT        = " + TotalSamples);
                    File.WriteLine("NCHAN       = " + NumChannels);
                    File.WriteLine("NORTH_ROT   = " + Rotation);
                    File.WriteLine("SENSOR_TYPE = " + TransducerType);
                    File.WriteLine("UNITS       = " + OutDataFormat);
                    File.WriteLine("CH0_ID      = " + IdChannel1);
                    File.WriteLine("CH1_ID      = " + IdChannel2);
                    File.WriteLine("CH2_ID      = " + IdChannel3);
                    File.WriteLine("####--------------------------------");
                }
                // --------------------------------------------------------------------- Voltage Coefficient 
                float kVoltage = 0;
                switch (OutDataFormat)
                {
                    case "mV":
                        kVoltage = 1f / VoltageGain;
                        // with min/max = -1650/+1650
                        break;
                    case "uV":
                        kVoltage = 1000f / VoltageGain;
                        // with min/max = -1650/+1650
                        break;
                }
 
                // --------------------------------------------------------------------- 
                string FormatString = "+0.000000E+0;-0.000000E+0";
                x = 0;
                UpdatingGraph = true;
                MillisecondPrecision();
                ReadSamplesTotal = 0;
                ThisTime = 0;
                NextTime = 0;
                AcquisitionTimer.Reset();
                AcquisitionTimer.Start();
                // --------------------------------------------------------------- do the acquisition loop
                do
                {
                    Percentual = Convert.ToInt32(ThisTime * 100 / AcquisitionTime_ms);
                    ReadSamplesTotal += 1;
                    //
                    for (i = 1; i <= NumChannels; i++)
                    {
                        LastValue[i] = ThereminoSlots.ReadSlot_NoNan(i) - MidValue[i];

                        if (WriteDataFile) 
                        {
                            File.Write((K[i] * LastValue[i]).ToString(FormatString,
                                                                      Module_Utils.GCI) + Constants.vbTab);
                        }
                    }
                    if (WriteDataFile)
						File.Write(File.NewLine);
					//
					NextTime += SamplingTime_ms;
					do {
						System.Threading.Thread.Sleep(1);
						ThisTime = AcquisitionTimer.Elapsed.TotalMilliseconds;
					} while (!(ThisTime >= NextTime));
					//
				} while (!(ReadSamplesTotal >= TotalSamples | StopAcquisitionFlag));
				// --------------------------------------------------------------- 
				AcquisitionTimer.Stop();
				UpdatingGraph = false;
				// --------------------------------------------------------------- 
				if (StopAcquisitionFlag) 
                {
					StopAcquisitionFlag = false;
					if (WriteDataFile) 
                    {
                        File.Write("# " + Utils_LocaleNames.Msg_AcquInterrupted);
					}
                    Interaction.MsgBox(Utils_LocaleNames.Msg_AcquInterrupted, MsgBoxStyle.SystemModal | MsgBoxStyle.Information, "");
				} 
                else 
                {
					double ElapsedMillisec = AcquisitionTimer.ElapsedMilliseconds / 1000;
                    Interaction.MsgBox(Utils_LocaleNames.Msg_Time + ": " + ElapsedMillisec.ToString() + Constants.vbCr + Constants.vbCr + Utils_LocaleNames.Msg_Samples + ": " + ReadSamplesTotal.ToString() + Constants.vbCr + Constants.vbCr + Utils_LocaleNames.Msg_Frequency + ": " + (ReadSamplesTotal / ElapsedMillisec).ToString() + "                  ", MsgBoxStyle.SystemModal | MsgBoxStyle.Information, "");
				}
				// --------------------------------------------------------------- 
				Operation = 0;
			} 
            catch 
            {
                Interaction.MsgBox(Utils_LocaleNames.Msg_OutputFileError, MsgBoxStyle.Critical, "");
			}
			try 
            {
				if (WriteDataFile) 
                {
					File.Close();
				}
			} 
            catch 
            {
			}
			// ---------------------------------------------------------------- to be sure in case of error catch
			AcquisitionTimer.Stop();
			NormalPrecision();
			Percentual = 0;
			Operation = 0;
			UpdatingGraph = false;
		}


        // ================================================================
        //   Set the K ARRAY with gains for all the channels from 1 to 24
        // ================================================================ 
        private float kVoltage;
        internal float k_HV = 1;
        internal float k_NE = 1;
        private float[] K = new float[25];

        private void SetChannelGains()
        {
            //  --------------------------------------------------------------------- Voltage Coefficient 
            // ............................................................
            //   Starting from version 5.3 values from slots are in "Volt"
            //   With Adc24 set min/max = -3.3/+3.3
            //   With Adc16 set min/max =   0 /+3.3
            // ............................................................
            switch (OutDataFormat)
            {
                case "mV":
                    kVoltage = 1000.0F / VoltageGain;
                    break;
                case "uV":
                    kVoltage = 1000000.0F / VoltageGain;
                    break;
            }
            //  --------------------------------------------------------------------- K for all the channels
            for (Int32 i = 0; (i <= 24); i++)
            {
                K[i] = kVoltage;
            }
            //  --------------------------------------------------------------------- HV trimming coefficients
            GetHvTrimmingCoefficient(IdChannel1);
            GetHvTrimmingCoefficient(IdChannel2);
            GetHvTrimmingCoefficient(IdChannel3);
        }

        //  ------------------------------------------------------------------------- H/V and breeze coefficients
        private float GetHvTrimmingCoefficient(string IdChannel)
        {
            float KV = 1;
            float KN;
            float KE;
            //  -------------------------------------------
            KN = 1 / Convert.ToSingle(Conversion.Val(cmb_BreezeAttenuation.Text));
            KN *= k_HV;
            KE = KN;
            KN *= k_NE;
            KE /= k_NE;
            switch (IdChannel)
            {
                case "V": return KV;
                case "N": return KN;
                case "E": return KE;
                default: return 1;
            }
        }

        private void cmb_HVScale_DropDownClosed(object sender, EventArgs e)
        {
            k_HV *= Convert.ToSingle(Conversion.Val(cmb_HVScale.Text));
            ShowKvalues();
            cmb_HVScale.Text = "1";
        }

        private void cmb_NEScale_DropDownClosed(object sender, EventArgs e)
        {
            k_NE *= Convert.ToSingle(Conversion.Val(cmb_NEScale.Text));
            ShowKvalues();
            cmb_NEScale.Text = "1";
        }

        private void ShowKvalues()
        {
            lbl_TrimHV.Text = "K = " + k_HV.ToString("0.000", Module_SaveLoad.GCI);
            lbl_TrimNE.Text = "K = " + k_NE.ToString("0.000", Module_SaveLoad.GCI);
        }

    }
}








