using Microsoft.VisualBasic;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using System.Reflection;

static class Module_SaveLoad
{
    public static string ApplicationName
    {
        get
        {
            var entryAssembly = Assembly.GetEntryAssembly();
            var applicationTitle = ((AssemblyTitleAttribute)entryAssembly.GetCustomAttributes(typeof(AssemblyTitleAttribute), false)[0]).Title;
            if (string.IsNullOrEmpty(applicationTitle))
            {
                applicationTitle = entryAssembly.GetName().Name;
            }
            return applicationTitle;
        }
    } 

	static internal bool EventsAreEnabled = false;

    static internal System.Globalization.CultureInfo GCI = System.Globalization.CultureInfo.InvariantCulture;

	// =======================================================================================================
	//   APP TITLE AND VERSION
	// =======================================================================================================
    public static string AppTitleAndVersion([System.Runtime.InteropServices.OptionalAttribute, System.Runtime.InteropServices.DefaultParameterValueAttribute("")]  // ERROR: Optional parameters aren't supported in C#
string Title)
    {
        if (string.IsNullOrEmpty(Title)) Title = ApplicationName.Replace('_', ' ');
        string[] s = Theremino_Dolfrang.My.MyProject.Application.Info.Version.ToString().Split('.');
        return Title + " - V" + s[0] + "." + s[1];
    }

    //  =======================================================================================
    //    FILES
    //  =======================================================================================
    public static bool FileExists(string fileName)
    {
        return System.IO.File.Exists(fileName);
    }



	// =======================================================================================
	//  FORM FUNCTIONS
	// =======================================================================================
	static internal void LimitFormPosition(System.Windows.Forms.Form f)
	{
		if (f.WindowState != FormWindowState.Normal)
			return;
		GetMaxScreenBounds();
		EnsureFormVisible(f);
		//EnsureFormCompletelyVisible(f)
	}


	private static Rectangle SB = new Rectangle(int.MaxValue, int.MaxValue, int.MinValue, int.MinValue);
	private static void GetMaxScreenBounds()
	{
		foreach (Screen s in System.Windows.Forms.Screen.AllScreens) {
			SB = Rectangle.Union(SB, s.WorkingArea);
		}
	}

	//Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
	//    With frm
	//        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
	//        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
	//        .Width = Math.Max(.Width, 32)               ' at least 32x24
	//        .Height = Math.Max(.Height, 24)             ' at least 32x24
	//        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
	//        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
	//        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
	//        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
	//    End With
	//End Sub

	private static void EnsureFormVisible(Form frm)
	{
		var _with1 = frm;
		_with1.Width = Math.Min(_with1.Width, SB.Width);
		// not more than VIRTUALSCREEN dimensions
		_with1.Height = Math.Min(_with1.Height, SB.Height);
		// not more than VIRTUALSCREEN dimensions 
		_with1.Width = Math.Max(_with1.Width, 32);
		// at least 32x24
		_with1.Height = Math.Max(_with1.Height, 24);
		// at least 32x24
		_with1.Left = Math.Min(_with1.Left, SB.Right - 50);
		// not beyond right border - 50 pixels
		_with1.Top = Math.Min(_with1.Top, SB.Bottom - 100);
		// not beyond bottom border - 50 pixels
		_with1.Left = Math.Max(_with1.Left, SB.Left + 100 - _with1.Width);
		// at least at left border + 50 pixels
		_with1.Top = Math.Max(_with1.Top, SB.Top - 10);
		// at least at top border
	}

	// (The value of the RestoreBounds property is valid only 
	//   when the WindowState property of the Form class is not equal to Normal)
	static internal Rectangle GetFormRectangle(Form frm)
	{
		Rectangle r = default(Rectangle);
		if (frm.WindowState == FormWindowState.Normal) {
			r = frm.Bounds;
		} else {
			r = frm.RestoreBounds;
		}
		return r;
	}


	// ================================================================================================
	//  Private Read-Write functions
	// ================================================================================================
    private static string TabString(string Name, double Value, string fmt)
    {
        Int32 nTab = Math.Max(0, 22 - Name.Length);
        if (double.IsNaN(Value))
        {
            return Name;
        }
        else
        {
            return Name + "=" + Strings.StrDup(nTab, " ") + Value.ToString(fmt);
        }
    }
	private static string TabString(string Name, bool Value)
	{

		Int32 nTab = Math.Max(0, 22 - Name.Length);

		return Name + "=" + Strings.StrDup(nTab, " ") + Value.ToString();
	}
	private static string TabString(string Name, string Value)
	{

		Int32 nTab = Math.Max(0, 22 - Name.Length);

		return Name + "=" + Strings.StrDup(nTab, " ") + Value;
	}
	private static float Val_Single(string l)
	{
		return Convert.ToSingle(Conversion.Val(l.Replace(",", ".")));
	}
	private static double Val_Double(string l)
	{
		return Conversion.Val(l.Replace(",", "."));
	}
	private static Int32 Val_Int(string l)
	{
		return Convert.ToInt32(Conversion.Val(l));
	}
    public static string ExtractParamName(ref string s)
    {
        string functionReturnValue = null;
        // ------------------------- Returns the first field from begin to the first "=" symbol
        // -------------------------  and removes it from the string
        int i = 0;
        i = Strings.InStr(s, "=", Microsoft.VisualBasic.CompareMethod.Binary);
        if (i > 0)
        {
            functionReturnValue = Strings.Trim(Strings.Left(s, i - 1));
            s = Strings.Trim(Strings.Mid(s, i + 1));
        }
        else
        {
            functionReturnValue = Strings.Trim(s);
            s = "";
        }
        return functionReturnValue;
    }
	private static string AssemblyName()
	{
		return System.Reflection.Assembly.GetExecutingAssembly().GetName().Name;
	}


	// ==================================================================================================
	//  SAVE LOAD -- Program INI
	// ==================================================================================================
	static internal void Save_INI()
	{
		// -------------------------------------------------------------------------------- 
        string iniFileName = Theremino.PlatformAdjustedFileName(Application.StartupPath + "\\" + AssemblyName() + "_INI.txt", "");
        System.IO.StreamWriter f = null;
		try {
			f = System.IO.File.CreateText(iniFileName);
			//
			f.WriteLine(" Program Params");
			f.WriteLine("===========================================");
			// ------------------------------------------------------------------------------ FORM BOUNDS
			Rectangle r = default(Rectangle);
			r = GetFormRectangle(Theremino_Dolfrang.My.MyProject.Forms.Form_Main);
            f.WriteLine(TabString("Form1_Top", r.Top, ""));
            f.WriteLine(TabString("Form1_Left", r.Left, ""));
            f.WriteLine(TabString("Form1_Width", r.Width, ""));
            f.WriteLine(TabString("Form1_Height", r.Height, ""));
            f.WriteLine(TabString("Form1_WindowState", (int)Theremino_Dolfrang.My.MyProject.Forms.Form_Main.WindowState, ""));
            //
            f.WriteLine(TabString("Language", Utils_LocaleNames.Language));
			//f.WriteLine(TabString("WriteDataFile", Form_Main.ToolStripButton_WriteDataFile.Checked))
			//
			f.WriteLine("");
            f.WriteLine(TabString(" Location params",0, ""));
			f.WriteLine("===========================================");
            f.WriteLine(TabString("SurveyName", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_SurveyName.Text));
            f.WriteLine(TabString("Department", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Department.Text));
            f.WriteLine(TabString("Locality", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Locality.Text));
            f.WriteLine(TabString("Notes", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Notes.Text));
			//
			f.WriteLine(TabString("", ""));
			f.WriteLine(TabString(" Other params", ""));
			f.WriteLine("===========================================");
            f.WriteLine(TabString("Rotation", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Rotation.Text));
            f.WriteLine(TabString("StationCode", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_StationCode.Text));
            f.WriteLine(TabString("VoltageGain", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_VoltageGain.Text));
            f.WriteLine(TabString("OutDataFormat", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_OutDataFormat.SelectedIndex, ""));
            f.WriteLine(TabString("TransducerType", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_TransducerType.SelectedIndex, ""));
            f.WriteLine(TabString("Channel1", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_Channel1.SelectedIndex, ""));
            f.WriteLine(TabString("Channel2", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_Channel2.SelectedIndex, ""));
            f.WriteLine(TabString("Channel3", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_Channel3.SelectedIndex, ""));
			//
            f.WriteLine(TabString("", ""));
            f.WriteLine(TabString(" Sampling params", ""));
			f.WriteLine("===========================================");
            f.WriteLine(TabString("SamplingFreq", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_SamplingFreq.SelectedIndex, ""));
            f.WriteLine(TabString("NumChannels", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_NumChannels.SelectedIndex, ""));
            f.WriteLine(TabString("ZeroingTime", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_ZeroingTime.SelectedIndex, ""));
            f.WriteLine(TabString("AcquisitionTimeUnit", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_AcquisitionTimeUnit.SelectedIndex, ""));
            f.WriteLine(TabString("AcquisitionTime", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_AcquisitionTime.SelectedIndex, ""));
			//
            f.WriteLine(TabString("", ""));
            f.WriteLine(TabString(" Visualization params", ""));
			f.WriteLine("===========================================");
            f.WriteLine(TabString("VisualizationFreq", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_VisualizationFreq.SelectedIndex, ""));
            f.WriteLine(TabString("ZoomX", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_ZoomX.SelectedIndex, ""));
            f.WriteLine(TabString("ZoomY", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_ZoomY.SelectedIndex, ""));
            //
            f.WriteLine(TabString("", ""));
            f.WriteLine(TabString(" Trimming", ""));
            f.WriteLine("===========================================");
            f.WriteLine(TabString("K_HV", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.k_HV, ""));
            f.WriteLine(TabString("K_NE", Theremino_Dolfrang.My.MyProject.Forms.Form_Main.k_NE, ""));
		} catch {
		}
		try {
			f.Close();
		} catch {
		}
	}

	static internal void Load_INI()
	{
		// -------------------------------------------------------------------------------- Defaults 
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.ToolStripButton_WriteDataFile.Checked = true;
		//
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_SurveyName.Text = "- - -";
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Department.Text = "";
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Locality.Text = "";
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Notes.Text = "";
		//
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_TransducerType.SelectedIndex = 1;
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_Channel1.SelectedIndex = 0;
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_Channel2.SelectedIndex = 1;
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_Channel3.SelectedIndex = 2;
		//
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Rotation.Text = "0";
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_StationCode.Text = "- - -";
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_VoltageGain.Text = "10000";
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_OutDataFormat.SelectedIndex = 0;
		//
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_SamplingFreq.SelectedIndex = 2;
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_NumChannels.SelectedIndex = 21;
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_ZeroingTime.SelectedIndex = 2;
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_AcquisitionTimeUnit.SelectedIndex = 0;
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_AcquisitionTime.SelectedIndex = 0;
		//
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_VisualizationFreq.SelectedIndex = 0;
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_ZoomX.SelectedIndex = 4;
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_ZoomY.SelectedIndex = 4;
        //
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_BreezeAttenuation.SelectedIndex = 6;
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_HVScale.SelectedIndex = 10;
        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_NEScale.SelectedIndex = 10;
		// ------------------------------------------------------------------------------- 
		// With "Resume Next" subsequent parameters are loaded and f.Close() is executed
		// -------------------------------------------------------------------------------
		 // ERROR: Not supported in C#: OnErrorStatement

		// use Resume-Next instead of Try-Catch
		// -------------------------------------------------------------------------------
		string l = null;
        string iniFileName = Theremino.PlatformAdjustedFileName(Application.StartupPath + "\\" + AssemblyName() + "_INI.txt", "");
		if (Theremino_Dolfrang.My.MyProject.Computer.FileSystem.FileExists(iniFileName)) {
			//
			System.IO.StreamReader f = null;
			f = System.IO.File.OpenText(iniFileName);
			//
			while (!f.EndOfStream) {
				l = f.ReadLine();
				switch (ExtractParamName(ref l)) {
					case "Form1_Top":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.Top = Val_Int(l);
						break;
					case "Form1_Left":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.Left = Val_Int(l);
						break;
					case "Form1_Width":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.Width = Val_Int(l);
						break;
					case "Form1_Height":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.Height = Val_Int(l);
						break;
					case "Form1_WindowState":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.WindowState = (FormWindowState)(Conversion.Val(l));
						break;
					// ------------------------------------------------------------------------- 
					case "Language":
                        Utils_LocaleNames.Language = l;
						break;
					//Case "WriteDataFile" : Form_Main.ToolStripButton_WriteDataFile.Checked = l = "True"
					// ------------------------------------------------------------------------- Survey data
					case "SurveyName":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_SurveyName.Text = l;
						break;
					case "Department":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Department.Text = l;
						break;
					case "Locality":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Locality.Text = l;
						break;
					case "Notes":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Notes.Text = l;
						break;
					// ------------------------------------------------------------------------- Hardware device
					case "Rotation":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_Rotation.Text = l;
						break;
					case "StationCode":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_StationCode.Text = l;
						break;
					case "VoltageGain":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.txt_VoltageGain.Text = l;
						break;
					case "OutDataFormat":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_OutDataFormat, Val_Int(l));
						break;
					case "TransducerType":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_TransducerType, Val_Int(l));
						break;
					case "Channel1":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_Channel1, Val_Int(l));
						break;
					case "Channel2":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_Channel2, Val_Int(l));
						break;
					case "Channel3":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_Channel3, Val_Int(l));
						break;
					// ------------------------------------------------------------------------- Acquisition
					case "SamplingFreq":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_SamplingFreq, Val_Int(l));
						break;
					case "NumChannels":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_NumChannels, Val_Int(l));
						break;
					case "ZeroingTime":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_ZeroingTime, Val_Int(l));
						break;
					case "AcquisitionTimeUnit":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_AcquisitionTimeUnit, Val_Int(l));
						break;
					case "AcquisitionTime":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_AcquisitionTime, Val_Int(l));
						break;
					// ------------------------------------------------------------------------- Visualization
					case "VisualizationFreq":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_VisualizationFreq, Val_Int(l));
						break;
					case "ZoomX":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_ZoomX, Val_Int(l));
						break;
					case "ZoomY":
                        Combo_SetIndex(Theremino_Dolfrang.My.MyProject.Forms.Form_Main.cmb_ZoomY, Val_Int(l));
						break;
                    // ------------------------------------------------------------------------- HV_Trimming
                    case "K_HV":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.k_HV = Val_Single(l);
                        break;
                    case "K_NE":
                        Theremino_Dolfrang.My.MyProject.Forms.Form_Main.k_NE = Val_Single(l);
                        break;
				}
			}
			f.Close();
		}
        LimitFormPosition(Theremino_Dolfrang.My.MyProject.Forms.Form_Main);
	}


	// ==============================================================================================================
	//   COMBO FUNCTIONS
	// ==============================================================================================================
	static internal void Combo_Init(ComboBox combo, string str)
	{
		bool old = EventsAreEnabled;
		EventsAreEnabled = false;
		if (str == null)
			str = "";
		var _with2 = combo;
		_with2.Items.Clear();
		_with2.Items.Add(str);
		_with2.SelectedIndex = 0;
		EventsAreEnabled = old;
	}
	static internal void Combo_SetIndex_FromString(ComboBox combo, string str)
	{
		bool old = EventsAreEnabled;
		EventsAreEnabled = false;
		if (str == null)
			str = "";
		var _with3 = combo;
		for (Int32 i = 0; i <= _with3.Items.Count - 1; i++) {
			if (_with3.Items[i].ToString() == str) {
				_with3.SelectedIndex = i;
				break;
			}
		}
		EventsAreEnabled = old;
	}
	static internal string Combo_GetValue(ComboBox combo)
	{
		if (combo.SelectedIndex < 0)
			return "";
		return combo.Items[combo.SelectedIndex].ToString();
	}
	static internal void Combo_SetIndex(ComboBox combo, Int32 index)
	{
		if (combo.Items.Count < 1)
			return;
		if (index < 0)
			index = 0;
		if (index > combo.Items.Count - 1)
			index = combo.Items.Count - 1;
		combo.SelectedIndex = index;
	}

	// =======================================================================================================
	//   FadeIn and FadeOut 
	// =======================================================================================================
	static internal void Forms_FadeTo(double FinalValue, double TimeMillisec)
	{
		if (TimeMillisec < 1)
			TimeMillisec = 1;
		double v = 0;
		double k = 0;
		System.DateTime date1 = default(System.DateTime);
		//
        double StartValue = Theremino_Dolfrang.My.MyProject.Forms.Form_Main.Opacity;
		//
		Application.DoEvents();
		System.Threading.Thread.Sleep(1);
		date1 = System.DateTime.Now;
		do {
			k = System.DateTime.Now.Subtract(date1).TotalMilliseconds / TimeMillisec;
			if (k > 1)
				k = 1;
			v = StartValue + (FinalValue - StartValue) * k;
			if (FinalValue == 0) {
				v = v * 0.5;
			}
            Theremino_Dolfrang.My.MyProject.Forms.Form_Main.Opacity = v;
			System.Threading.Thread.Sleep(20);
			//Debug.Print(v.ToString)
		} while (!(k >= 1));
	}

}
